/* Copyright (C) 2022 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Sergei Zharko [committer], Maksym Zyzak */

/// \file    CaTrackFinder.h

#pragma once  // include this header only once per compilation unit

#include "CaBranch.h"
#include "CaHit.h"
#include "CaSimd.h"
#include "CaTrackParam.h"
#include "CaVector.h"


namespace cbm::algo::ca
{

  class Track;
  class Triplet;
  class Framework;

  namespace
  {
    using namespace cbm::algo;  // to keep ca:: prefices in the code
  }

  /// Class implements a clones merger algorithm for the CA track finder
  ///
  class TrackFinder {
   public:
    /// Default constructor
    TrackFinder(ca::Framework& algo);

    /// Destructor
    ~TrackFinder();

    /// Copy constructor
    TrackFinder(const TrackFinder&) = delete;

    /// Move constructor
    TrackFinder(TrackFinder&&) = delete;

    /// Copy assignment operator
    TrackFinder& operator=(const TrackFinder&) = delete;

    /// Move assignment operator
    TrackFinder& operator=(TrackFinder&&) = delete;

    void FindTracks();
    void Init();

   private:
    // -------------------------------
    // Private methods

    bool checkTripletMatch(const ca::Triplet& l, const ca::Triplet& r, fscal& dchi2) const;


   private:
    // -------------------------------
    // Data members

    ca::Framework& frAlgo;  ///< Reference to the main track finder algorithm class

    std::vector<float> fvTsStartThread;
    std::vector<float> fvTsEndThread;
    std::vector<float> fvWindowStartThread;

    float fWindowLength  = 0.;
    float fWindowOverlap = 15.;  // ns
  };

}  // namespace cbm::algo::ca
